import pandas as pd
import FeatureSelection.features as features
import FeatureSelection.select_point_feature as point_feature
import configparser

CONFIG = configparser.ConfigParser()
CONFIG.read('../config.ini')


def same_class_diff(df, output_path, size):
    oil_ids = df.oil_id.unique()

    if size < 0:
        size = len(oil_ids)//2
    if size > len(oil_ids)//2:
        size = len(oil_ids) // 2
    
    for oil_id in range(size):
        id1 = oil_id * 2
        id2 = id1 + 1
        df1 = df.loc[df['oil_id'] == oil_ids[id1]]
        df2 = df.loc[df['oil_id'] == oil_ids[id2]]

        map = features.diff_map(df1, df2, CONFIG['columns']['v_start'], CONFIG['columns']['v_end'])
        file_name = output_path + "diff_map_" + str(id1) + "-" + str(id2) + ".csv"
        map.to_csv(file_name, header=True, index=False)


def different_class_diff(source1, source2, output_path, size):
    oil_ids1 = source1.oil_id.unique()
    oil_ids2 = source2.oil_id.unique()

    # if size of sample is not defined, compare all oils from smaller class
    if size < 0:
        if len(oil_ids1) < len(oil_ids2):
            size = len(oil_ids1)
        else:
            size = len(oil_ids2)

    for oil_id in range(size):
        df1 = source1.loc[source1['oil_id'] == oil_ids1[oil_id]]
        df2 = source2.loc[source2['oil_id'] == oil_ids2[oil_id]]

        map = features.diff_map(df1, df2, CONFIG['columns']['v_start'], CONFIG['columns']['v_end'])
        file_name = output_path + "diff_map_" + str(oil_id) + ".csv"
        map.to_csv(file_name, header=True, index=False)


def create_template(source, output_file):
    oil_ids = source.oil_id.unique()
    first = True
    number_of_maps = 0

    for oil_id in range(len(oil_ids)):
        number_of_maps += 1
        df = source.loc[source['oil_id'] == oil_ids[oil_id]]

        # separate columns
        start_value = df.columns.get_loc(CONFIG['columns']['v_start'])
        end_value = df.columns.get_loc(CONFIG['columns']['v_end']) + 1
        df = df[df.columns[start_value:end_value]].reset_index()

        # sum
        if first:
            map_sum = df
            first = False
        else:
            map_sum = map_sum + df
    # average = sum/number of oils
    map_sum /= number_of_maps
    map_sum = map_sum.drop('index', axis='columns')
    map_sum.to_csv(output_file, header=True, index=False)


def template_diff(source1, template, output_path, size):
    template = pd.read_csv(template)
    oil_ids1 = source1.oil_id.unique()

    # if size of sample is not defined, compare all oils from smaller class
    if size < 0:
        size = len(oil_ids1)

    for oil_id in range(size):
        df1 = source1.loc[source1['oil_id'] == oil_ids1[oil_id]]
        map_diff = features.diff_map(df1, template, CONFIG['columns']['v_start'], CONFIG['columns']['v_end'])
        file_name = output_path + "diff_map_" + str(oil_id) + ".csv"
        map_diff.to_csv(file_name, header=True, index=False)


def create_mask(source_file, template, output_path):
    points = point_feature.getDiffPoints(source_file, float(CONFIG['diff_maps']['THRESHOLD']))
    # read from template for same size
    mask = pd.read_csv(template)

    # set all values to 1 (default)
    for col in mask.columns:
        mask[col].values[:] = 1

    # for each point set value to 0 - places where are anomalies in class
    for index, row in points.iterrows():
        start_row = row["start index"]
        end_row = row["end index"]
        start_v = mask.columns.get_loc(row["start v"])
        end_v = mask.columns.get_loc(row["end v"]) + 1
        mask.loc[start_row:end_row, mask.columns[start_v:end_v]] = 0

    mask.to_csv(output_path)


if __name__ == "__main__":
    source = pd.read_csv(CONFIG['diff_maps']['RAW_FILE_TRAIN'])
    evoo = source.loc[source["class"] == "EVOO"]
    loo = source.loc[source["class"] == "LOO"]
    voo = source.loc[source["class"] == "VOO"]
    print("diff map for different class")
    different_class_diff(evoo, loo, CONFIG['diff_maps']['DIFF_MAP_PATH_EVOO_LOO'], -1)
    different_class_diff(evoo, voo, CONFIG['diff_maps']['DIFF_MAP_PATH_EVOO_VOO'], -1)
    different_class_diff(loo, voo, CONFIG['diff_maps']['DIFF_MAP_PATH_LOO_VOO'], -1)
    print("create template")
    create_template(evoo, CONFIG['diff_maps']['TEMPLATE_FILE_EVOO'])
    create_template(voo, CONFIG['diff_maps']['TEMPLATE_FILE_VOO'])
    create_template(loo, CONFIG['diff_maps']['TEMPLATE_FILE_LOO'])
    print("teamplate diff")
    template_diff(evoo, CONFIG['diff_maps']['TEMPLATE_FILE_EVOO'], CONFIG['diff_maps']['TEMPLATE_PATH_EVOO_EVOO'], -1)
    template_diff(voo, CONFIG['diff_maps']['TEMPLATE_FILE_VOO'], CONFIG['diff_maps']['TEMPLATE_PATH_VOO_VOO'], -1)
    template_diff(loo, CONFIG['diff_maps']['TEMPLATE_FILE_LOO'], CONFIG['diff_maps']['TEMPLATE_PATH_LOO_LOO'], -1)
    print("mask")
    create_mask(CONFIG['diff_maps']['TEMPLATE_PATH_EVOO_EVOO'], CONFIG['diff_maps']['TEMPLATE_FILE_EVOO'], CONFIG['diff_maps']['MASK_FILE_EVOO'])
    create_mask(CONFIG['diff_maps']['TEMPLATE_PATH_VOO_VOO'], CONFIG['diff_maps']['TEMPLATE_FILE_VOO'], CONFIG['diff_maps']['MASK_FILE_VOO'])
    create_mask(CONFIG['diff_maps']['TEMPLATE_PATH_LOO_LOO'], CONFIG['diff_maps']['TEMPLATE_FILE_LOO'], CONFIG['diff_maps']['MASK_FILE_LOO'])
