import pandas as pd
from PIL import Image
import math
import os
import configparser

CONFIG = configparser.ConfigParser()
CONFIG.read('../config.ini')


def cut_columns(df):
    result = df.set_index('spectrum')
    start = result.columns.get_loc(CONFIG['columns']['v_start'])
    result = result.iloc[:, start:]

    return result


def linear_interpolation_color(value):
    f = float(value - float(CONFIG['pictures']['min_value'])) / (float(CONFIG['pictures']['max_value']) - float(CONFIG['pictures']['min_value']))
    if f > 1:
        f = 1
    elif f < 0:
        f = 0
    red = int(f * 255)
    blue = 255-red

    return red, blue


def create_image(oil_df, directory, oil_id):
    im = Image.new("RGB", (int(CONFIG['pictures']['width']), int(CONFIG['pictures']['height'])))
    pix = im.load()

    for x in range(int(CONFIG['pictures']['height'])):
        try:
            row = oil_df.loc[x]
            if isinstance(row, pd.DataFrame):
                row = row.iloc[0]
            row_length = len(row)
            for y in range(int(CONFIG['pictures']['width'])):
                if y <= row_length:
                    if math.isnan(row[y]):
                        pix[y, x] = (0, 0, 0)
                    else:
                        red, blue = linear_interpolation_color(row[y])
                        pix[y, x] = (red, 0, blue)
                else:
                    pix[y, x] = (0, 0, 0)
        except KeyError:
            for y in range(int(CONFIG['pictures']['width'])):
                pix[y, x] = (0, 0, 0)

    file = CONFIG['pictures']['pictures_dir'] + directory + '/' + oil_id + '.png'
    im.save(file, "PNG")


def prepare_dataframe_for_picture():
    if not os.path.exists(CONFIG['pictures']['input_file']):
        print('Subor nebol najdeny')
    else:
        df = pd.read_csv(CONFIG['pictures']['input_file'])

        uniq_ids = df.oil_id.unique()
        for oil_id in uniq_ids:
            oil_df_all = df[df['oil_id'] == oil_id]

            directory = oil_df_all.iloc[0]['class']
            oil_df_cut = cut_columns(oil_df_all)

            create_image(oil_df_cut, directory, oil_id)


if __name__ == "__main__":
    for oil_dir in CONFIG['pictures']['classes'].split(','):
        if not os.path.exists(CONFIG['pictures']['pictures_dir'] + oil_dir):
            os.makedirs(CONFIG['pictures']['pictures_dir'] + oil_dir)

    prepare_dataframe_for_picture()
