import sys
import numpy as np
import pandas as pd
from sklearn.feature_selection import RFE
from sklearn.tree import DecisionTreeClassifier
from sklearn.model_selection import train_test_split

CLASS_TO_LABEL = {
    'EVOO': 0, 'VOO': 1, 'LOO': 2,
}


def filter_method(df):
    cor = df.corr()

    cor_target = abs(cor["class"])

    relevant_features = cor_target[cor_target > 0.5]
    print(type(relevant_features))

    for feature in relevant_features.keys():
        print(feature)


def print_rfe_features(df, labels, nof):

    df = df.drop('class', axis=1)
    model = DecisionTreeClassifier()
    df_train, df_test, labels_train, labels_test = train_test_split(df, labels, test_size=0.20)
    cols = list(df.columns)

    # Initializing RFE model
    rfe = RFE(estimator=model, n_features_to_select=nof)
    # Transforming data using RFE
    x_train_rfe = rfe.fit_transform(df_train, labels_train)
    x_test_rfe = rfe.transform(df_test)
    # Fitting the data to model
    model.fit(x_train_rfe, labels_train)

    temp = pd.Series(rfe.support_, index=cols)
    selected_features_rfe = temp[temp == True].index

    print("[", end='')
    for feature in selected_features_rfe:
        print("'{}', ".format(feature), end='')
    print("]")

    score = model.score(x_test_rfe, labels_test)
    print("Score with {} features: {}".format(nof, score))


def recursive_feature_elimination(df, labels):
    bottom_border = 20
    upper_border = 501
    nof_list = np.arange(bottom_border, upper_border)
    high_score = 0
    # Variable to store the optimum features
    nof = 0
    score_list = []

    # dropping class column
    df = df.drop('class', axis=1)

    for n in range(len(nof_list)):
        model = DecisionTreeClassifier()
        df_train, df_test, labels_train, labels_test = train_test_split(df, labels, test_size=0.20)
        # Initializing RFE model
        print(n + bottom_border)
        rfe = RFE(estimator=model, n_features_to_select=n + bottom_border)
        # Transforming data using RFE
        x_train_rfe = rfe.fit_transform(df_train, labels_train)
        x_test_rfe = rfe.transform(df_test)
        # Fitting the data to model
        model.fit(x_train_rfe, labels_train)

        score = model.score(x_test_rfe, labels_test)
        print(score)
        score_list.append(score)
        if score > high_score:
            high_score = score
            nof = nof_list[n]

    print("Optimum number of features: %d" % nof)
    print("Score with %d features: %f" % (nof, high_score))


if __name__ == "__main__":

    df = pd.read_csv(sys.argv[1])

    # retyping column class from string type to int type
    for index, row in df.iterrows():
        df.loc[index, 'class'] = CLASS_TO_LABEL[row['class']]

    df['class'] = df['class'].astype(int)

    label = df['class']
    df = df.drop('oil_id', axis=1)

    filter_method(df)
    recursive_feature_elimination(df, label)
    print_rfe_features(df, label, 102)
