import pandas as pd
import os


def apply_threshold(data, threshold):
    for col in data.columns:
        data.loc[data[col] < threshold, col] = 0
    return data


def get_points(data):
    points = pd.DataFrame({"start index": [], "end index": [], "start v": [], "end v": [],
                           "max": [], "mean": []})
    indexes = data.loc[data.sum(axis=1) > 0].index
    length = len(indexes)
    first = True
    start = -1
    end = -1
    for i in indexes:
        if first:
            start = i
            end = i
            first = False
        elif i == end + 1:
            end = i
        else:
            selected = data.iloc[start:end + 1]
            selected = selected[selected.columns[selected.sum(axis=0) != 0]]
            if len(selected.columns) > 0:
                points = points.append({"start index": start, "end index": end,
                                        "start v": selected.columns[0], "end v": selected.columns[-1],
                                        "max": selected.values.max(), "mean": selected.values.mean()},
                                       ignore_index=True)
                start = i
                end = i
            else:
                print("Error: no values found")

    #     last anomaly
    if start > 0 and end > 0:
        selected = data.iloc[start:end + 1]
        selected = selected[selected.columns[selected.sum(axis=0) != 0]]
        if len(selected.columns) > 0:
            points = points.append({"start index": start, "end index": end,
                                    "start v": selected.columns[0], "end v": selected.columns[-1],
                                    "max": selected.values.max(), "mean": selected.values.mean()}, ignore_index=True)
    return points


def average_diff_maps(maps_path):
    files = os.listdir(maps_path)
    first = True
    number_of_maps = 0

    for file_name in files:
        if ".csv" in file_name:
            number_of_maps += 1
            file_path = maps_path + "/" + file_name
            diffMap = pd.read_csv(file_path)
            diffMap = diffMap.abs()
            if first:
                map_sum = diffMap
                first = False
            else:
                map_sum = map_sum + diffMap
    map_sum /= number_of_maps
    return map_sum


def getDiffPoints(path, threshold):
    averaged = average_diff_maps(path)
    data_threshold = apply_threshold(averaged, threshold)
    return get_points(data_threshold)


def get_point_feature_name(path, max_mean_diff):
    files = os.listdir(path)
    for file_name in files:
        file_path = path + "/" + file_name
        points = pd.read_csv(file_path)
        select = points.loc[points["max"]-points["mean"] < max_mean_diff]
        for index,row in select.iterrows():
            v, value = row["start v"].split("_")
            v, value_end = row["end v"].split("_")
            value_len = int(value_end) - int(value)
            spectrum = row["start index"] + 1
            spectrum_len = row["end index"] - row["start index"]
            print("'point_"+value
                  + "-" + str(value_len)
                  + "-" + str(int(spectrum))
                  + "-" + str(int(spectrum_len)), end="', ")


if __name__ == "__main__":
    evoo = getDiffPoints("../data/Feature/DiffMaps/templates/EVOO", 2)
    print(evoo)
    # evoo.to_csv("data/Feature/DiffMaps/points/evoo.csv", index=False)
