from torch.utils.data import Dataset
from os import listdir
from os.path import join
import torch
import cv2 as cv
import torchvision.transforms as transforms

CLASS_TO_LABEL = {
    'evoo': 0, 'voo': 1, 'loo': 2,
}


def read_image(path):
    image = cv.imread(path)
    image = cv.cvtColor(image, cv.COLOR_BGR2RGB)
    return image


def read_image_chw(path):
    image = read_image(path)
    image = image.transpose((2, 0, 1))
    return image


class ImageDataset(Dataset):
    def __init__(self, dataset_dir):
        self.dataset_dir = dataset_dir
        self.y = []
        self.image_paths = []

        for class_name in CLASS_TO_LABEL.keys():
            class_label = CLASS_TO_LABEL[class_name]
            class_image_paths = list(listdir(join(dataset_dir, class_name)))
            self.y += [class_label] * len(class_image_paths)
            self.image_paths += (join(dataset_dir, class_name, x) for x in class_image_paths)

        self.num_samples = len(self.y)
        self.images = [None] * len(self)

    def __len__(self):
        return self.num_samples

    def __getitem__(self, idx):
        if self.images[idx] is None:
            image = read_image_chw(self.image_paths[idx])
            image = torch.tensor(image).float()
            transform = transforms.Compose([transforms.Resize((224, 224))])  # For changing resize of image
            image = transform(image)
            image = (image - image.mean()) / image.std()
            self.images[idx] = image
        x = self.images[idx]

        y = self.y[idx]
        y = torch.tensor(y).long()

        return x, y
