import torch
import torch.nn as nn


def conv_block(in_channels, out_channels, kernel, padding):
    layer = nn.Sequential(
        nn.Conv2d(in_channels, out_channels, kernel_size=kernel, padding=padding),
        nn.BatchNorm2d(out_channels),
        nn.ReLU()
    )
    return layer


def vgg_fc_layer(in_channels, out_channels,):
    layer = nn.Sequential(
        nn.Linear(in_channels, out_channels),
        nn.ReLU()
    )
    return layer


class Vgg16(nn.Module):
    def __init__(self, num_classes):
        super(Vgg16, self).__init__()

        # Conv blocks
        self.layer0 = conv_block(3, 64, 3, 1)
        self.layer1 = conv_block(64, 64, 3, 1)
        self.maxpool0 = nn.MaxPool2d(2, 2)

        self.layer2 = conv_block(64, 128, 3, 1)
        self.layer3 = conv_block(128, 128, 3, 1)
        self.maxpool1 = nn.MaxPool2d(2, 2)

        self.layer4 = conv_block(128, 256, 3, 1)
        self.layer5 = conv_block(256, 256, 3, 1)
        self.layer6 = conv_block(256, 256, 3, 1)
        self.maxpool2 = nn.MaxPool2d(2, 2)

        self.layer7 = conv_block(256, 512, 3, 1)
        self.layer8 = conv_block(512, 512, 3, 1)
        self.layer9 = conv_block(512, 512, 3, 1)
        self.maxpool3 = nn.MaxPool2d(2, 2)

        self.layer10 = conv_block(512, 512, 3, 1)
        self.layer11 = conv_block(512, 512, 3, 1)
        self.layer12 = conv_block(512, 512, 3, 1)
        self.maxpool4 = nn.MaxPool2d(2, 2)

        # FC layers
        self.layer13 = vgg_fc_layer(7 * 7 * 512, 4096)  # change in case different resize of image
        self.layer14 = vgg_fc_layer(4096, 4096)

        # Final layer
        self.layer15 = nn.Linear(4096, num_classes)

    def forward(self, x):
        out = self.layer0(x)
        out = self.layer1(out)
        out = self.maxpool0(out)

        out = self.layer2(out)
        out = self.layer3(out)
        out = self.maxpool1(out)

        out = self.layer4(out)
        out = self.layer5(out)
        out = self.layer6(out)
        out = self.maxpool2(out)

        out = self.layer7(out)
        out = self.layer8(out)
        out = self.layer9(out)
        out = self.maxpool3(out)

        out = self.layer10(out)
        out = self.layer11(out)
        out = self.layer12(out)
        out = self.maxpool4(out)

        out = out.reshape(out.shape[0], -1)

        out = self.layer13(out)
        out = self.layer14(out)
        out = self.layer15(out)

        return out
