import pandas as pd
import numpy as np
import math
import os
import configparser

CONFIG = configparser.ConfigParser()
CONFIG.read('../config.ini')

DF_FILE = CONFIG['missing_spectra']['input_file']
DF_OUTPUT_FILE = CONFIG['missing_spectra']['output_file']


def add_missing_spectra(df):
    df['diff'] = df.groupby(['oil_id'])['analysis_time'].diff()
    unique_values = df['diff'].unique()

    for x in unique_values:
        x = round(x, 2)
        if x != 0.13 and math.isnan(x) is False:
            idx = pd.MultiIndex.from_product([df['oil_id'].unique(),
                                              np.arange(0.13, df['analysis_time'].max() + 0.13, 0.13)],
                                             names=['oil_id', 'analysis_time'])

            df = df.set_index(['oil_id', 'analysis_time']).reindex(idx).reset_index()
            count = 1
            oil_id = df.loc[0, 'oil_id']

            for index, row in df.iterrows():
                if oil_id == df.loc[index, 'oil_id']:
                    df.loc[index, 'spectrum'] = count
                else:
                    oil_id = df.loc[index, 'oil_id']
                    count = 1
                    df.loc[index, 'spectrum'] = count
                count += 1

    df = df.drop('diff', axis=1)

    return df


if __name__ == "__main__":
    if os.path.exists(CONFIG['missing_spectra']['input_file']):
        file = pd.read_csv(CONFIG['missing_spectra']['input_file'])
        data = add_missing_spectra(file)
        data.to_csv(CONFIG['missing_spectra']['output_file'], mode='w', header=True, index=False)
    else:
        print("File does not exist.")
