import re


def print_header_unsorted(output):
    header = "class,oil_id,spectrum,drift_field_intensity,pressure,temperature,drift_tube_length,analysis_time"
    for x in range(449):
        header += ',t_' + str(x) + ',v_' + str(x)
    output.write(header+"\n")


def print_header(output):
    header = "class,oil_id,spectrum,drift_field_intensity,pressure,temperature,drift_tube_length,analysis_time"
    for x in range(449):
        header += ',v_' + str(x)
    output.write(header+"\n")


def print_header_df():
    header = ["class", "oil_id", "spectrum", "drift_field_intensity", "pressure", "temperature", "drift_tube_length",
              "analysis_time"]
    for x in range(449):
        value = "v_" + str(x)
        header.append(value)

    return header


def add_spectrum(head, line):
    test_format = re.search('Spectrum [0-9]+\\.\\n', line)
    if not test_format:
        print("Error: spectrum format not supported")
        return head + ","
    splitted = re.split(" ", line)
    spectrum = splitted[-1]

    # delete .\n
    if spectrum[-2] == '.':
        spectrum = spectrum[:-2]
    else:
        print("Warning: spectrum has wrong format")
    head += ',' + spectrum
    return head


def add_attribute(head, line):
    test_format = re.search(".* [0-9,\\.]+ \\[.*\\]\\n", line)
    if not test_format:
        print("Error: attribute format not supported")
        return head + ","
    splitted = re.split(" ", line)
    attribute = splitted[-2]

    test_format = re.search("^\\-?[0-9]+\\.?[0-9]*$", attribute)
    if not test_format:
        print("Error: attribute not number")
        return head + ","

    head += ',' + attribute
    return head


# used only in unsorted
def add_value(head, line):
    test_format = re.search("^[ ]*\\-?[0-9]+[\\.]*[0-9]*[ ]+\\-?[0-9]+[\\.]*[0-9]*\\n", line)
    if not test_format:
        print("Error: time and value have wrong format")
        return head + ",,"
    splitted = re.split(" ", line)
    splitted = list(filter(None, splitted))

    value = splitted[1]
    value = value[:-1]

    test_format = re.search("^\\-?[0-9]+\\.?[0-9]*$", splitted[0])
    if not test_format:
        print("Error: time is not number")
        return head + ",,"
    test_format = re.search("^\\-?[0-9]+\\.?[0-9]*$", value)
    if not test_format:
        print("Error: value is not number")
        return head + ",,"

    head += ',' + splitted[0] + ',' + value
    return head


def get_time_value(line):
    test_format = re.search("^[ ]*\\-?[0-9]+[\\.]*[0-9]*[ ]+\\-?[0-9]+[\\.]*[0-9]*\\n", line)
    if not test_format:
        print("Error: time and value have wrong format")
        return None
    splitted = re.split(" ", line)
    splitted = list(filter(None, splitted))

    value = splitted[1]
    value = value[:-1]  # cut end of line symbol

    test_format = re.search("^\\-?[0-9]+\\.?[0-9]*$", splitted[0])
    if not test_format:
        print("Error: time is not number")
        return None
    test_format = re.search("^\\-?[0-9]+\\.?[0-9]*$", value)
    if not test_format:
        print("Error: value is not number")
        return None
    time = splitted[0]

    return time, value


def convert_oil(file_name, output, head):

    f = open(file_name, "r")
    line = f.readline()
    while True:
        row = head
        if not line:
            break
        # spectrum
        row = add_spectrum(row, line)

        # attributes 5x (Drift field intensity,Pressure,Temperature T1,Drift tube length,Analysis time)
        for x in range(5):
            line = f.readline()
            row = add_attribute(row, line)

        # skip mobility line
        line = f.readline()

        # values
        number_of_values = 0
        values = ""
        while True:
            line = f.readline()
            if not line or line[0] == 'S':  # EOF or new spectrum
                break

            time, value = get_time_value(line)
            number_of_values += 1
            values += ',' + value

        # different number of values
        if number_of_values < 449:
            row += values + ','
        else:
            row += values

        if number_of_values < 448 or number_of_values > 449:
            print("warning: different number of values ", number_of_values)
        output.write(row + "\n")
    f.close()


def convert_oil_to_df(df, file_name, head):

    f = open(file_name, "r")
    line = f.readline()

    while True:
        row = head
        if not line:
            break

        # spectrum
        row = add_spectrum(row, line)

        # attributes 5x (Drift field intensity,Pressure,Temperature T1,Drift tube length,Analysis time)
        for x in range(5):
            line = f.readline()
            row = add_attribute(row, line)

        # skip mobility line
        line = f.readline()

        # values
        number_of_values = 0
        values = ""
        while True:
            line = f.readline()
            if not line or line[0] == 'S':  # EOF or new spectrum
                break

            time, value = get_time_value(line)
            number_of_values += 1
            values += ',' + value

        # different number of values
        if number_of_values < 449:
            row += values + ','
        else:
            row += values

        if number_of_values < 448 or number_of_values > 449:
            print("warning: different number of values ", number_of_values)
        row = row.split(',')
        df.loc[len(df)] = row
    f.close()

    return df
