import numpy as np
from pickle import dump
from pickle import load
import pandas as pd
from sklearn import preprocessing
import os
import configparser

CONFIG = configparser.ConfigParser()
CONFIG.read('../config.ini')


def get_min_max_values(df1):
    df_values = df1.values
    min_value = np.nanmin(df_values)
    max_value = np.nanmax(df_values)
    print(min_value, ' ', max_value)
    return min_value, max_value


def cut_v_columns(df):
    v_0 = df.columns.get_loc("v_0")
    df1 = df.iloc[:, v_0:]
    print('cut')
    return df1


def clip_values(df1):
    df1 = df1.clip(lower=0, upper=1)
    print('clip')
    return df1


def delete_v_448_column(df):
    try:
        df.columns.get_loc("v_448")
        df = df.drop(columns=['v_448'])
    except:
        pass

    print('delete')
    return df


def normalize_fit(df1):
    min_value, max_value = get_min_max_values(df1)

    min_max_scaler = preprocessing.MinMaxScaler()
    min_max_value = np.array([[min_value], [max_value]])
    min_max_scaler.fit(min_max_value)

    # save the scaler
    dump(min_max_scaler, open(CONFIG['normalization']['scaler_file'], 'wb'))
    print('fit')


def normalize_transform(df1):
    # load the scaler
    if os.path.exists(CONFIG['normalization']['scaler_file']):
        min_max_scaler = load(open(CONFIG['normalization']['scaler_file'], 'rb'))

        x = df1.values
        x_scaled = min_max_scaler.transform(x)
        df1 = pd.DataFrame(x_scaled, columns=df1.columns)
        df1 = clip_values(df1)

        print('transform')
        return df1
    else:
        print("SCALER_FILE does not exist.")
        return None


def normalize(df):
    df1 = cut_v_columns(df)  # create dataframe of value columns only

    normalize_fit(df1)
    df1 = normalize_transform(df1)

    df[df1.columns] = df1

    df = delete_v_448_column(df)

    return df


# only transform - for app usage
def transform_only(df):
    df1 = cut_v_columns(df)
    df1 = normalize_transform(df1)
    if df1 is None:
        return None
    else:
        df[df1.columns] = df1
        return df


if __name__ == "__main__":
    if os.path.exists(CONFIG['normalization']['input_file']):
        df = pd.read_csv(CONFIG['normalization']['input_file'])
        # data = normalize(df)
        data = transform_only(df)
        data.to_csv(CONFIG['normalization']['output_file'], mode='w', header=True, index=False)
    else:
        print("File does not exist.")
