import os
import pandas as pd
from Predprocessing.check_missing_spectra import add_missing_spectra
from Predprocessing.convert_functions import convert_oil_to_df, print_header_df
from Predprocessing.filter_values import filter_values_last, filter_values_1_150
from Predprocessing.normalization import normalize, transform_only
import configparser

CONFIG = configparser.ConfigParser()
CONFIG.read('../config.ini')

DF_FILE = CONFIG['predprocess_data']['input_file']
DF_OUTPUT_FILE = CONFIG['predprocess_data']['output_file']


def predprocess(oil_type, files, train, id):
    for file in files:
        column_names = print_header_df()
        head = oil_type + "," + str(id)
        df = pd.DataFrame(columns=column_names)
        data = convert_oil_to_df(df, file, head)

        data[["oil_id", "spectrum", "drift_field_intensity", "pressure", "temperature", "drift_tube_length",
             "analysis_time"]] = data[["oil_id", "spectrum", "drift_field_intensity", "pressure", "temperature",
                                      "drift_tube_length", "analysis_time"]].apply(pd.to_numeric)

        for x in range(449):
            value = "v_" + str(x)
            data[[value]] = data[[value]].apply(pd.to_numeric)

        data = add_missing_spectra(data)
        data = filter_values_1_150(data)
        data = filter_values_last(data)

        if train is True:
            data = normalize(data)
        else:
            data = transform_only(data)

        file_path = CONFIG['predprocess_data']['output_file'] + id + ".csv"

        data.to_csv(file_path, mode='w', header=True, index=False)
        id += 1


if __name__ == "__main__":
    files = []
    file_name = str(CONFIG['predprocess_data']['input_file'])
    files.append(file_name)
    oil_type = 'EVOO'
    id = 0
    if os.path.exists(CONFIG['predprocess_data']['input_file']):
        predprocess(oil_type, files, True, id)
    else:
        print("File does not exist.")
