from sklearn.model_selection import GridSearchCV
from joblib import dump, load
import os.path
import pandas as pd
from pandas import np
from sklearn.model_selection import train_test_split
from sklearn.tree import DecisionTreeClassifier
from Statistic.evaluation import evaluate_clf


def train_decision_tree(train_features, train_labels, tuning=False):
    if tuning:
        best_params = decision_tree_tuning(train_features, train_labels)
        clf = DecisionTreeClassifier()
        clf.set_params(**best_params)
    else:
        selected_params = {'ccp_alpha': 0.0, 'class_weight': None, 'criterion': 'gini',
                           'max_depth': None, 'max_features': None, 'max_leaf_nodes': None,
                           'min_impurity_decrease': 0.0, 'min_impurity_split': None,
                           'min_samples_leaf': 10, 'min_samples_split': 2, 'min_weight_fraction_leaf': 0.0,
                           'presort': 'deprecated', 'random_state': None, 'splitter': 'random'
                           }
        clf = DecisionTreeClassifier()
        clf.set_params(**selected_params)
    return clf


def decision_tree_tuning(train_features, train_labels):
    tuned_parameters = {
        'criterion': ['gini', 'entropy'],
        'splitter': ['best', 'random'],
        'max_depth': [None, 3, 5, 7, 10, 15, 20, 30, 50, 100, 150, 200],
        'min_samples_split': [int(x) for x in np.linspace(start=2, stop=100, num=2)],
        'min_samples_leaf': [1, 2, 3, 5, 10, 20, 30, 50]
    }

    clf = GridSearchCV(DecisionTreeClassifier(), tuned_parameters, cv=5, verbose=1, scoring='accuracy')
    clf.fit(train_features, train_labels)

    print("Best parameters set found on development set:")
    print(clf.best_params_)
    print("skore")
    print(clf.best_score_)
    return clf.best_params_


def classify_decision_tree(df):
    labels = np.array(df['class'])
    df = df.drop('class', axis=1)
    df = df.drop('oil_id', axis=1)
    features_array = np.array(df)

    train_features, test_features, train_labels, test_labels = train_test_split(features_array, labels, test_size=0.20)

    if not os.path.exists('decision_tree.joblib'):
        # tarining
        clf = DecisionTreeClassifier()

    else:

        clf = load('decision_tree.joblib')

    clf.fit(train_features, train_labels)
    pred_labels = clf.predict(test_features)
    dump(clf, 'decision_tree.joblib')

    return evaluate_clf(test_labels, pred_labels, "decision_tree")


if __name__ == "__main__":
    df = pd.read_csv('../Data/Train/Feature/featuresAll.csv')
    classify_decision_tree(df)
