from sklearn.model_selection import GridSearchCV
from joblib import dump, load
import os.path
import pandas as pd
from pandas import np
from sklearn.model_selection import train_test_split
from sklearn.neighbors import KNeighborsClassifier
from Statistic.evaluation import evaluate_clf


def train_knn(train_features, train_labels, tuning=False):
    if tuning:
        best_params = knn_tuning(train_features, train_labels)
        clf = KNeighborsClassifier()
        clf.set_params(**best_params)
    else:
        selected_params = {'algorithm': 'auto', 'leaf_size': 10, 'metric': 'manhattan',
                           'metric_params': None, 'n_jobs': None, 'n_neighbors': 11,
                           'p': 2, 'weights': 'uniform'
                           }
        clf = KNeighborsClassifier()
        clf.set_params(**selected_params)
    return clf


def knn_tuning(train_features, train_labels):
    tuned_parameters = {
        'n_neighbors': [3,5,11,19],
        'weights': ['uniform', 'distance'],
        'metric': ['euclidean','manhattan'],
        'algorithm': ['auto', 'ball_tree', 'kd_tree', 'brute'],
        'leaf_size': [int(x) for x in np.linspace(start=10, stop=100, num=10)]
    }

    clf = GridSearchCV(KNeighborsClassifier(), tuned_parameters, cv=5, verbose=1, scoring='accuracy')
    clf.fit(train_features, train_labels)

    print("Best parameters set found on development set:")
    print(clf.best_params_)
    print("skore")
    print(clf.best_score_)
    return clf.best_params_


def classify_knn(df):
    labels = np.array(df['class'])
    df = df.drop('class', axis=1)
    df = df.drop('oil_id', axis=1)
    features_array = np.array(df)

    train_features, test_features, train_labels, test_labels = train_test_split(features_array, labels, test_size=0.20)

    if not os.path.exists('knn.joblib'):
        # tarining
        clf = KNeighborsClassifier()

    else:

        clf = load('knn.joblib')

    clf.fit(train_features, train_labels)
    pred_labels = clf.predict(test_features)
    dump(clf, 'knn.joblib')

    return evaluate_clf(test_labels, pred_labels, "knn")


if __name__ == "__main__":
    df = pd.read_csv('../Data/Train/Feature/featuresAll.csv')
    classify_knn(df)
