from sklearn.model_selection import GridSearchCV
from joblib import dump, load
import os.path
import pandas as pd
from pandas import np
from sklearn.model_selection import train_test_split
from sklearn.naive_bayes import GaussianNB
from Statistic.evaluation import evaluate_clf


def train_naive_bayes(train_features, train_labels, tuning=False):
    if tuning:
        best_params = naive_bayes_tuning(train_features, train_labels)
        clf = GaussianNB()
        clf.set_params(**best_params)
    else:
        selected_params = {'priors': None, 'var_smoothing': 1e-09}
        clf = GaussianNB()
        clf.set_params(**selected_params)
    return clf


def naive_bayes_tuning(train_features, train_labels):
    tuned_parameters = {
        'var_smoothing': np.logspace(0,-9, num=100)
    }

    clf = GridSearchCV(GaussianNB(), tuned_parameters, cv=5, verbose=1, scoring='accuracy')
    clf.fit(train_features, train_labels)

    print("Best parameters set found on development set:")
    print(clf.best_params_)
    print("skore")
    print(clf.best_score_)
    return clf.best_params_


def classify_naive_bayes(df):
    labels = np.array(df['class'])
    df = df.drop('class', axis=1)
    df = df.drop('oil_id', axis=1)
    features_array = np.array(df)

    train_features, test_features, train_labels, test_labels = train_test_split(features_array, labels, test_size=0.20)

    if not os.path.exists('naive_bayes.joblib'):
        # tarining
        clf = GaussianNB()

    else:

        clf = load('naive_bayes.joblib')

    clf.fit(train_features, train_labels)
    pred_labels = clf.predict(test_features)
    dump(clf, 'naive_bayes.joblib')

    return evaluate_clf(test_labels, pred_labels, "naive_bayes")

if __name__ == "__main__":
    df = pd.read_csv('../Data/Train/Feature/featuresAll.csv')
    classify_naive_bayes(df)
