import os
import numpy as np
import pandas as pd
from joblib import load
from Statistic.create_chart import scores_charts
from Statistic.evaluation import evaluate_clf

import operator


def predict_class(classifier, test_features):
    model_name = classifier
    classifier = classifier.split("-")[0]
    path = '../Models/StandardClasification/' + model_name

    if not os.path.exists(path):
        print("The path is not existing.")
        return None
    else:
        clf = load(path)
    pred_labels = clf.predict(test_features)

    return pred_labels


def combine_predictions(predictions_arr):
    final_arr = []
    for index in range(len(predictions_arr[0])):
        values = {'EVOO': 0,
                  'VOO': 0,
                  'LOO': 0}

        for clasif_predict in predictions_arr:
            values[clasif_predict[index]] += 1
        clasified_value = max(values.items(), key=operator.itemgetter(1))[0]
        final_arr.append(clasified_value)

    final_arr = np.array(final_arr)
    return final_arr


def validation(classifiers, data):
    test_labels = np.array(data['class'])
    data = data.drop('class', axis=1)
    data = data.drop('oil_id', axis=1)
    test_features = np.array(data)

    scores = []
    predictions_arr = []
    for i in range(len(classifiers)):
        classifier = classifiers[i]
        predictions = predict_class(classifier, test_features)
        predictions_arr.append(predictions)
        score = evaluate_clf(test_labels, predictions, classifier)
        scores.append(score)

    if len(classifiers) > 1:
        combinated_predict_arr = combine_predictions(predictions_arr)
        score = evaluate_clf(test_labels, combinated_predict_arr, 'combinated')
        scores.append(score)

    scores_charts(scores)


def predict(classifiers, data):
    data = data.drop('oil_id', axis=1)
    test_features = np.array(data)

    predictions_arr = []
    predictions = None
    for i in range(len(classifiers)):
        classifier = classifiers[i]
        predictions = predict_class(classifier, test_features)
        predictions_arr.append(predictions)

    if len(classifiers) > 1:
        combinated_predict_arr = combine_predictions(predictions_arr)
        return combinated_predict_arr, predictions_arr
    else:
        return predictions, predictions_arr


if __name__ == "__main__":
    # Testing
    data = pd.read_csv('../Data/Test/Feature/featuresTest.csv')
    classifiers = ['decision_tree-2020-12-07 01-23-17.joblib', 'random_forest-2020-12-07 00-09-57.joblib']
    validation(classifiers, data)
