from sklearn.model_selection import GridSearchCV
from joblib import dump, load
import os.path
import pandas as pd
from pandas import np
from sklearn.model_selection import train_test_split
from sklearn.ensemble import RandomForestClassifier
from Statistic.evaluation import evaluate_clf


def train_random_forest(train_features, train_labels, tuning=False):
    if tuning:
        best_params = rf_tuning(train_features, train_labels)
        clf = RandomForestClassifier()
        clf.set_params(**best_params)
    else:
        selected_params = {'bootstrap': True, 'ccp_alpha': 0.0, 'class_weight': None, 'criterion': 'gini',
                           'max_depth': None, 'max_features': 'auto', 'max_leaf_nodes': None, 'max_samples': None,
                           'min_impurity_decrease': 0.0, 'min_impurity_split': None, 'min_samples_leaf': 1,
                           'min_samples_split': 2, 'min_weight_fraction_leaf': 0.0, 'n_estimators': 100, 'n_jobs': None,
                           'oob_score': False, 'random_state': None, 'verbose': 0, 'warm_start': False
                           }
        clf = RandomForestClassifier()
        clf.set_params(**selected_params)
    return clf


def rf_tuning(train_features, train_labels):
    tuned_parameters = {
        'n_estimators': [int(x) for x in np.linspace(start=10, stop=160, num=20)],
        'criterion': ['gini', 'entropy'],
        'min_samples_split': [int(x) for x in np.linspace(start=2, stop=30, num=2)],
        'bootstrap': [True, False],
        'max_depth': [None, 5, 10, 20],
        'max_leaf_nodes': [None, 5, 10, 15],
        'max_samples': [None, 5, 10, 15]

    }

    clf = GridSearchCV(RandomForestClassifier(), tuned_parameters, cv=5, verbose=1, scoring='accuracy')
    clf.fit(train_features, train_labels)

    print("Best parameters set found on development set:")
    print(clf.best_params_)
    print(clf.best_score_)
    return clf.best_params_


def classify_random_forest(df):
    labels = np.array(df['class'])
    df = df.drop('class', axis=1)
    df = df.drop('oil_id', axis=1)
    features_array = np.array(df)

    train_features, test_features, train_labels, test_labels = train_test_split(features_array, labels, test_size=0.20)

    if not os.path.exists('random_forest.joblib'):
        # tarining
        clf = RandomForestClassifier()

    else:

        clf = load('random_forest.joblib')

    clf.fit(train_features, train_labels)
    pred_labels = clf.predict(test_features)
    dump(clf, 'random_forest.joblib')

    return evaluate_clf(test_labels, pred_labels, "random_forest")


if __name__ == "__main__":
    df = pd.read_csv('../data/Feature/featuresAll-point.csv')
    classify_random_forest(df)
