from sklearn.model_selection import GridSearchCV
from joblib import dump, load
import os.path
import pandas as pd
from pandas import np
from sklearn.model_selection import train_test_split
from sklearn import svm
from Statistic.evaluation import evaluate_clf


def train_svm(train_features, train_labels, tuning=False):
    if tuning:
        best_params = svm_tuning(train_features, train_labels)
        clf = svm.SVC()
        clf.set_params(**best_params)
    else:
        selected_params = {'C': 1.0, 'break_ties': False, 'cache_size': 200, 'class_weight': None,
                           'coef0': 0.0, 'decision_function_shape': 'ovr', 'degree': 3, 'gamma': 'scale',
                           'kernel': 'rbf', 'max_iter': -1, 'probability': False, 'random_state': None,
                           'shrinking': True, 'tol': 0.001, 'verbose': False
                           }
        clf = svm.SVC()
        clf.set_params(**selected_params)
    return clf


def svm_tuning(train_features, train_labels):
    tuned_parameters = {'kernel': ['rbf', 'linear'],
                        'gamma': [1e-3, 1e-4, 'auto', 'scale'],
                        'decision_function_shape': ['ovo','ovr'],
                         'C': [1, 10, 20, 50, 70, 100]}

    clf = GridSearchCV(svm.SVC(), tuned_parameters, cv=5, verbose=1, scoring='accuracy')
    clf.fit(train_features, train_labels)

    print("Best parameters set found on development set:")
    print(clf.best_params_)
    print(clf.best_score_)
    return clf.best_params_


def classify_svm(df):
    labels = np.array(df['class'])
    df = df.drop('class', axis=1)
    df = df.drop('oil_id', axis=1)
    features_array = np.array(df)

    train_features, test_features, train_labels, test_labels = train_test_split(features_array, labels, test_size=0.20)

    if not os.path.exists('svm.joblib'):
        # tarining
        clf = svm.SVC()

    else:

        clf = load('svm.joblib')

    clf.fit(train_features, train_labels)
    pred_labels = clf.predict(test_features)
    dump(clf, 'svm.joblib')

    return evaluate_clf(test_labels, pred_labels, "svm")


if __name__ == "__main__":
    df = pd.read_csv('../Data/Train/Feature/featuresAll.csv')
    classify_svm(df)
