from StandardClasification.naive_bayes import classify_naive_bayes
from StandardClasification.decision_tree import classify_decision_tree
from StandardClasification.random_forest import classify_random_forest
from StandardClasification.knn import classify_knn
from StandardClasification.svm import classify_svm
import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.ticker as mtick
import os


def scores_charts(scores, file_name="score"):
    # Funkcia na vytvorenie grafu pre vizualizaciu skore pre skore klasifikatorov
    print('saving chart in scores_charts', file_name)
    scores = [list(x) for x in zip(*scores)]
    width = 0.2
    print(scores)
    plt.bar(np.arange(len(scores[0])), scores[0], width=width, label='accuracy', color='c', alpha=0.7)
    plt.bar(np.arange(len(scores[1])) + width, scores[1], width=width, label='F1 micro', color='m', alpha=0.7)
    plt.bar(np.arange(len(scores[2])) + width + width, scores[2], width=width, label='F1 macro', color='y', alpha=0.7)

    plt.xticks([r + width for r in range(5)], ['KNN', 'RF', 'SVC', 'DT', 'GB'])

    ax = plt.gca()

    ax.yaxis.set_major_formatter(mtick.PercentFormatter())
    ax.set_ylim(bottom=0, top=100)

    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc='lower left', ncol=3, mode="expand")

    plt.title('Uspesnost klasifikatorov', y=-0.15)
    plt.tight_layout()
    ax.yaxis.grid(True, linestyle='--')
    print('saving chart in scores_charts', file_name)
    plt.savefig(os.path.join('../Statistic/charts/', file_name + '.png'))
    plt.clf()


if __name__ == "__main__":
    df = pd.read_csv('../Data/Train/Feature/featuresAll.csv')
    scores = []

    scores.append(classify_knn(df))
    scores.append(classify_random_forest(df))
    scores.append(classify_svm(df))
    scores.append(classify_decision_tree(df))
    scores.append(classify_naive_bayes(df))

    scores_charts(scores)
