import pandas as pd
import math
import sys
sys.path.insert(1, '../Predprocessing/')
import convert_functions


class TestAddSpectrum:
    def test_one_digit(self):
        s = "Spectrum 1.\n"
        value = convert_functions.add_spectrum("", s)
        assert value == ",1"

    def test_head(self):
        s = "Spectrum 1.\n"
        value = convert_functions.add_spectrum("aa", s)
        assert value == "aa,1"

    def test_more_digit(self):
        s = "Spectrum 1234.\n"
        value = convert_functions.add_spectrum("", s)
        assert value == ",1234"

    def test_more_letters(self):
        s = "Spectrum 1234.aa\n"
        value = convert_functions.add_spectrum("", s)
        assert value == ","

    def test_missing_letters(self):
        s = "Spectrm 1234.\n"
        value = convert_functions.add_spectrum("", s)
        assert value == ","

    def test_missing_dot(self):
        s = "Spectrum 1234\n"
        value = convert_functions.add_spectrum("", s)
        assert value == ","

    def test_missing_eof(self):
        s = "Spectrum 1234."
        value = convert_functions.add_spectrum("", s)
        assert value == ","


class TestAddAttribute:
    def test_drift(self):
        s = "Drift field intensity: 558.521 [V/cm]\n"
        value = convert_functions.add_attribute("", s)
        assert value == ",558.521"

    def test_drift_no_dot(self):
        s = "Drift field intensity: 558521 [V/cm]\n"
        value = convert_functions.add_attribute("", s)
        assert value == ",558521"

    def test_att(self):
        s = "Temperature T1: 110.292 [C]\n"
        value = convert_functions.add_attribute("", s)
        assert value == ",110.292"

    def test_att_two_dots(self):
        s = "Temperature T1: 110.2.92 [C]\n"
        value = convert_functions.add_attribute("", s)
        assert value == ","

    def test_att_no_eof(self):
        s = "Temperature T1: 110.292 [C]"
        value = convert_functions.add_attribute("", s)
        assert value == ","

    def test_att_no_end(self):
        s = "Temperature T1: 110.2.92\n"
        value = convert_functions.add_attribute("", s)
        assert value == ","


class TestAddValue:
    def test_correct(self):
        s = " 24.43648      0.034\n"
        value = convert_functions.add_value("", s)
        assert value == ",24.43648,0.034"

    def test_head(self):
        s = " 24.43648      0.034\n"
        value = convert_functions.add_value("aa", s)
        assert value == "aa,24.43648,0.034"

    def test_minus(self):
        s = " -24.43648      -0.034\n"
        value = convert_functions.add_value("", s)
        assert value == ",-24.43648,-0.034"

    def test_no_space(self):
        s = "24.43648 0.034\n"
        value = convert_functions.add_value("", s)
        assert value == ",24.43648,0.034"

    def test_dots(self):
        s = " 24.43.648      0.034\n"
        value = convert_functions.add_value("", s)
        assert value == ",,"

    def test_no_spaces(self):
        s = "24.436480.034\n"
        value = convert_functions.add_value("", s)
        assert value == ",,"


class TestGetTimeValue:
    def test_correct(self):
        s = " 24.43648      0.034\n"
        value = convert_functions.get_time_value(s)
        assert value[0] == "24.43648" and value[1] == "0.034"

    def test_minus(self):
        s = " -24.43648      -0.034\n"
        value = convert_functions.get_time_value(s)
        assert value[0] == "-24.43648" and value[1] == "-0.034"

    def test_no_space(self):
        s = "24.43648 0.034\n"
        value = convert_functions.get_time_value(s)
        assert value[0] == "24.43648" and value[1] == "0.034"

    def test_dots(self):
        s = " 24.43.648      0.034\n"
        value = convert_functions.get_time_value(s)
        assert value is None

    def test_no_spaces(self):
        s = "24.436480.034\n"
        value = convert_functions.get_time_value(s)
        assert value is None
