import pandas as pd
import math
import sys
sys.path.insert(1, '../FeatureSelection/')
import features
import create_features_df


class TestMeanFeature:
    def test_mean_normal_all(self):
        d = {'v_0': [1, 4], 'v_1': [4, 7]}
        df = pd.DataFrame(data=d)
        value = features.mean(df, 'ALL')
        assert value == 4

    def test_mean_decimal_all(self):
        d = {'v_0': [1, 10], 'v_1': [3, 5]}
        df = pd.DataFrame(data=d)
        value = features.mean(df, 'ALL')
        assert value == 4.75

    def test_mean_nan_all(self):
        d = {'v_0': [math.nan, math.nan], 'v_1': [math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.mean(df, 'ALL')
        assert math.isnan(value) == True

    def test_mean_nan_column(self):
        d = {'v_0': [math.nan, math.nan], 'v_1': [math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.mean(df, 'v_0')
        assert math.isnan(value) == True
        
    def test_mean_column(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, math.nan, 8]}
        df = pd.DataFrame(data=d)
        value = features.mean(df, 'v_0')
        assert round(value, 2) == round(28/3, 2)

    def test_mean_wrong_string(self):
        d = {'v_0': [5, 9, 14], 'v_1': [1, 5, 224]}
        df = pd.DataFrame(data=d)
        value = features.mean(df, 'v2')
        assert value is None

    def test_mean_decimal(self):
        d = {'v_0': [5, 9.18, -14], 'v_1': [4, 4, -8]}
        df = pd.DataFrame(data=d)
        value = features.mean(df, 'v_0')
        assert round(value, 2) == 0.06
        
        
class TestFeaturesDF:
    def test_create_features_df(self):
        d_all = {'class': ['EVOO', 'EVOO', 'EVOO'], 'oil_id': ['1', '1', '2'], 'data' : ['1234', '2345', '3456']}
        df_all = pd.DataFrame(data=d_all)
        d_control = {'class': ['EVOO', 'EVOO'], 'oil_id': ['1', '2']}
        df_control = pd.DataFrame(data=d_control)
        df_features = create_features_df.create_features_df(df_all)
        assert df_features.equals(df_control) == True


class TestMaxFeature:
    def test_max(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, 4, 8]}
        df = pd.DataFrame(data=d)
        value = features.max(df, 'v_0')
        assert value == 14

    def test_max_decimal(self):
        d = {'v_0': [5, 9.14, -14], 'v_1': [4, 4, -8]}
        df = pd.DataFrame(data=d)
        value = features.max(df, 'v_0')
        assert value == 9.14

    def test_max_nan(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, math.nan, 8]}
        df = pd.DataFrame(data=d)
        value = features.max(df, 'v_1')
        assert value == 8

    def test_max_all_nan(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.max(df, 'v_1')
        assert math.isnan(value)

    def test_max_wrong_string(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.max(df, 'v2')
        assert value is None


class TestMinFeature:
    def test_min(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, 4, 8]}
        df = pd.DataFrame(data=d)
        value = features.min(df, 'v_0')
        assert value == 5

    def test_min_decimal(self):
        d = {'v_0': [5, 9.14, -14.2], 'v_1': [4, 4, -8]}
        df = pd.DataFrame(data=d)
        value = features.min(df, 'v_0')
        assert value == -14.2

    def test_min_nan(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, math.nan, 8]}
        df = pd.DataFrame(data=d)
        value = features.min(df, 'v_1')
        assert value == 4

    def test_min_all_nan(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.min(df, 'v_1')
        assert math.isnan(value)

    def test_min_wrong_string(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.min(df, 'v2')
        assert value is None


class TestMedianFeature:
    def test_median_odd(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, 4, 8]}
        df = pd.DataFrame(data=d)
        value = features.median(df, 'v_0')
        assert value == 9

    def test_median_even(self):
        d = {'v_0': [5, 9, 14, 45], 'v_1': [4, 4, 8, 7]}
        df = pd.DataFrame(data=d)
        value = features.median(df, 'v_0')
        assert value == 11.5

    def test_median_decimal(self):
        d = {'v_0': [5, 9.14, -14, 45], 'v_1': [4, 4, -8, 4.58]}
        df = pd.DataFrame(data=d)
        value = features.median(df, 'v_0')
        assert value == 7.07

    def test_median_decimal(self):
        d = {'v_0': [5, 9.14, 9.15, 45], 'v_1': [4, 4, -8, 4.58]}
        df = pd.DataFrame(data=d)
        value = features.median(df, 'v_0')
        assert value == 9.145

    def test_median_nan(self):
        d = {'v_0': [5, 9, 14, 4, -47], 'v_1': [4, math.nan, 8, 45, 7]}
        df = pd.DataFrame(data=d)
        value = features.median(df, 'v_1')
        assert value == 7.5

    def test_median_all_nan(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.median(df, 'v_1')
        assert math.isnan(value)

    def test_median_wrong_string(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.median(df, 'v2')
        assert value is None


class TestStdFeature:
    def test_std(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, 4, 8]}
        df = pd.DataFrame(data=d)
        value = features.std(df, 'v_0')
        assert round(value, 4) == 3.6818

    def test_std_decimal(self):
        d = {'v_0': [5, 9.14, -14], 'v_1': [4, 4, -8]}
        df = pd.DataFrame(data=d)
        value = features.std(df, 'v_0')
        assert round(value, 4) == 10.0753

    def test_std_nan(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, math.nan, 8]}
        df = pd.DataFrame(data=d)
        value = features.std(df, 'v_1')
        assert round(value, 4) == 2

    def test_std_all_nan(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.std(df, 'v_1')
        assert math.isnan(value)

    def test_std_wrong_string(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.std(df, 'v2')
        assert value is None


class TestDeltaFeature:
    def test_delta(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, 4, 8]}
        df = pd.DataFrame(data=d)
        value = features.delta(df, 'v_0')
        assert value == 9

    def test_delta_decimal(self):
        d = {'v_0': [5, 9.14, -14], 'v_1': [4, 4, -8]}
        df = pd.DataFrame(data=d)
        value = features.delta(df, 'v_0')
        assert value == 23.14

    def test_delta_nan(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, math.nan, 8]}
        df = pd.DataFrame(data=d)
        value = features.delta(df, 'v_1')
        assert value == 4

    def test_delta_all_nan(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.delta(df, 'v_1')
        assert math.isnan(value)

    def test_delta_wrong_string(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.delta(df, 'v2')
        assert value is None


class TestSumFeature:
    def test_sum(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, 4, 8]}
        df = pd.DataFrame(data=d)
        value = features.sum(df, 'v_0')
        assert value == 28

    def test_sum_negative(self):
        d = {'v_0': [5, -9, 14], 'v_1': [4, 4, 8]}
        df = pd.DataFrame(data=d)
        value = features.sum(df, 'v_0')
        assert value == 10

    def test_sum_decimal(self):
        d = {'v_0': [5, 9.14, -14], 'v_1': [4, 4, -8]}
        df = pd.DataFrame(data=d)
        value = features.sum(df, 'v_0')
        assert round(value, 4) == 0.14

    def test_sum_nan(self):
        d = {'v_0': [5, 9, 14], 'v_1': [4, math.nan, 8]}
        df = pd.DataFrame(data=d)
        value = features.sum(df, 'v_1')
        assert value == 12

    def test_sum_all_nan(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.sum(df, 'v_1')
        assert value == 0

    def test_sum_wrong_string(self):
        d = {'v_0': [5, 9, 14], 'v_1': [math.nan, math.nan, math.nan]}
        df = pd.DataFrame(data=d)
        value = features.sum(df, 'v2')
        assert value is None
