import pandas as pd
import math
import sys
sys.path.insert(1, '../FeatureSelection/')
import features
import create_features_df


class TestPointFeature:
    def test_point_max(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "max_2-1-3-1")
        assert value == 72

    def test_point_max_missing_columns(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_4": [1, 5, 224, -42, 8],
             "v_5": [2, 4, 22, 2, 78], "v_6": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "max_0-3-1-2")
        assert value == 224

    def test_point_max2(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "max_0-3-1-2")
        assert value == 224

    def test_point_max_na(self):
        d = {"spectrum": [1, 2, math.nan, 4, 5], "v_0": [math.nan, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, math.nan, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "max_0-3-1-2")
        assert math.isnan(value) is True

    def test_point_max_na2(self):
        d = {"spectrum": [math.nan, math.nan, math.nan, math.nan, math.nan],
             "v_0": [math.nan, math.nan, math.nan, math.nan, math.nan], "v_1": [math.nan, math.nan, math.nan, -42, 8],
             "v_2": [math.nan, math.nan, math.nan, 2, 78], "v_3": [-1, -1, 0, math.nan, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "max_0-3-1-2")
        assert math.isnan(value) is True

    def test_point_mean(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "mean_2-1-3-1")
        assert value == 24

    def test_point_mean2(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "mean_0-3-1-2")
        assert round(value, 4) == 23.6667

    def test_point_mean_na(self):
        d = {"spectrum": [1, 2, math.nan, 4, 5], "v_0": [math.nan, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, math.nan, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "mean_0-3-1-2")
        assert math.isnan(value) is True

    def test_point_mean_na2(self):
        d = {"spectrum": [math.nan, math.nan, math.nan, math.nan, math.nan],
             "v_0": [math.nan, math.nan, math.nan, math.nan, math.nan], "v_1": [math.nan, math.nan, math.nan, -42, 8],
             "v_2": [math.nan, math.nan, math.nan, 2, 78], "v_3": [-1, -1, 0, math.nan, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "mean_0-3-1-2")
        assert math.isnan(value) is True

    def test_point_format_function(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "meaan_0-3-1-2")
        assert value is None

    def test_point_format_outV(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "mean_0-10-1-2")
        assert value is None

    def test_point_format_outS(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "meaan_0-3-1-10")
        assert value is None

    def test_point_format(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "mean-0-3-1-10")
        assert value is None

    def test_point_format2(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "mean_0x3-1-10")
        assert value is None

    def test_point_format3(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "mean_0-3-1")
        assert value is None

    def test_point_format4(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_123": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_3": [-1, -1, 0, 72, 0]}
        df = pd.DataFrame(data=d)
        value = features.point(df, "mean_0-3-1-1")
        assert value is None


class TestDiffMap:
    def test_diff_map(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_447": [-1, -1, 0, 72, 0]}
        df1 = pd.DataFrame(data=d)
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 424, -42, 8],
             "v_2": [2, -5, 22, 2, 78], "v_447": [-1, -8, 0, 72, 0]}
        df2 = pd.DataFrame(data=d)
        d = {"v_0": [0, 0, 0, 0, 0], "v_1": [0, 0, -200, 0, 0],
             "v_2": [0, 9, 0, 0, 0], "v_447": [0, 7, 0, 0, 0]}
        df_result = pd.DataFrame(data=d)
        value = features.diff_map(df1, df2, "v_0", "v_447")
        diff = value - df_result   # difference between returned value and espected result
        assert diff.values.sum() == 0

    def test_diff_map_nan(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, math.nan, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_447": [-1, -1, 0, 72, 0]}
        df1 = pd.DataFrame(data=d)
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, math.nan, 14, 12, -3], "v_1": [1, math.nan, 424, -42, 8],
             "v_2": [2, -5, 22, 2, 78], "v_447": [-1, -8, 0, 72, 0]}
        df2 = pd.DataFrame(data=d)
        d = {"v_0": [0, 0, 0, 0, 0], "v_1": [0, 0, -200, 0, 0],
             "v_2": [0, 9, 0, 0, 0], "v_447": [0, 7, 0, 0, 0]}
        df_result = pd.DataFrame(data=d)
        value = features.diff_map(df1, df2, "v_0", "v_447")
        value = value.fillna(0)
        diff = value - df_result   # difference between returned value and espected result
        assert diff.values.sum() == 0

    def test_diff_map_wrong_col(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_447": [-1, -1, 0, 72, 0]}
        df1 = pd.DataFrame(data=d)
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 424, -42, 8],
             "v_2": [2, -5, 22, 2, 78], "v_447": [-1, -8, 0, 72, 0]}
        df2 = pd.DataFrame(data=d)
        value = features.diff_map(df1, df2, "v_0", "v447")
        assert value is None

    def test_diff_map_wrong_col2(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_447": [-1, -1, 0, 72, 0]}
        df1 = pd.DataFrame(data=d)
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 424, -42, 8],
             "v_2": [2, -5, 22, 2, 78], "v_447": [-1, -8, 0, 72, 0]}
        df2 = pd.DataFrame(data=d)
        value = features.diff_map(df1, df2, "v_0", "v_450")
        assert value is None

    def test_diff_map_wrong_order(self):
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_447": [-1, -1, 0, 72, 0]}
        df1 = pd.DataFrame(data=d)
        d = {"spectrum": [1, 2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 424, -42, 8],
             "v_2": [2, -5, 22, 2, 78], "v_447": [-1, -8, 0, 72, 0]}
        df2 = pd.DataFrame(data=d)
        d = {"v_0": [0, 0, 0, 0, 0], "v_1": [0, 0, -200, 0, 0],
             "v_2": [0, 9, 0, 0, 0], "v_447": [0, 7, 0, 0, 0]}
        df_result = pd.DataFrame(data=d)
        value = features.diff_map(df1, df2, "v_447", "v_0")
        diff = value - df_result  # difference between returned value and espected result
        assert diff.values.sum() == 0

    def test_diff_map_wrong_order(self):
        d = {"spectrum": [1, 2.2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 224, -42, 8],
             "v_2": [2, 4, 22, 2, 78], "v_447": [-1, -1, 0, 72, 0]}
        df1 = pd.DataFrame(data=d)
        d = {"spectrum": [1, 2.2, 3, 4, 5], "v_0": [5, 9, 14, 12, -3], "v_1": [1, 5, 424, -42, 8],
             "v_2": [2, -5, 22, 2, 78], "v_447": [-1, -8, 0, 72, 0]}
        df2 = pd.DataFrame(data=d)
        d = {"v_0": [0, 0, 0, 0, 0], "v_1": [0, 0, -200, 0, 0],
             "v_2": [0, 9, 0, 0, 0], "v_447": [0, 7, 0, 0, 0]}
        df_result = pd.DataFrame(data=d)
        value = features.diff_map(df1, df2, "v_447", "v_0")
        diff = value - df_result  # difference between returned value and espected result
        assert diff.values.sum() == 0
