import pandas as pd
import math
import sys

sys.path.append('../')
import Predprocessing.normalization as normalization  # Spustitelne z UnitTests


class TestSupportFunctions:
    df = pd.DataFrame(data={'t_0': [1, 4], 't_1': [4, 7], 'v_0': [1, 4], 'v_1': [4, 7]})
    df1 = pd.DataFrame(data={'v_0': [1, 4], 'v_1': [4, 7]})
    df2 = pd.DataFrame(data={'v_0': [1.3, 0.5], 'v_1': [1, math.nan]})
    df3 = pd.DataFrame(data={'v_0': [-1, 0.5], 'v_1': [1, 7]})
    df4 = pd.DataFrame(data={'v_0': [0, 0.5], 'v_1': [1, 1]})
    df5 = pd.DataFrame(data={'v_0': [-1, 0.5], 'v_1': [1, 7], 'v_448': [1, 7]})

    def test_cut(self):
        df_cut = normalization.cut_v_columns(self.df)
        assert df_cut.equals(self.df1)


    def test_min_max(self):
        min_control = 1
        max_control = 7
        min_value, max_value = normalization.get_min_max_values(self.df)
        assert min_value == min_control and max_value == max_control


    def test_min_max_with_nan(self):
        min_control = 0.5
        max_control = 1.3
        min_value, max_value = normalization.get_min_max_values(self.df2)
        assert min_value == min_control and max_value == max_control


    def test_clip(self):
        df_clip = normalization.clip_values(self.df3)
        assert df_clip.equals(self.df4)


    def test_delete_v_448(self):
        df_deleted = normalization.delete_v_448_column(self.df5)
        assert df_deleted.equals(self.df3)


class TestNormalization:
    df1 = pd.DataFrame(data={'t_0': [1, 4], 't_1': [4, 7], 'v_0': [0, 5], 'v_1': [2, 10]})
    df2 = pd.DataFrame(data={'t_0': [1, 4], 't_1': [4, 7], 'v_0': [0, 0.5], 'v_1': [0.2, 1]})
    df3 = pd.DataFrame(data={'t_0': [1, 4], 't_1': [4, 7], 'v_0': [0, 5], 'v_1': [math.nan, 10]})
    df4 = pd.DataFrame(data={'t_0': [1, 4], 't_1': [4, 7], 'v_0': [0, 0.5], 'v_1': [math.nan, 1]})
    df5 = pd.DataFrame(data={'t_0': [1, 4], 't_1': [4, 7], 'v_0': [0, 5], 'v_1': [2, 10], 'v_448': [math.nan, 7]})


    def test_normalization(self):
        assert normalization.normalize(self.df1).equals(self.df2)


    def test_normalization_nan(self):
        assert normalization.normalize(self.df3).equals(self.df4)


    def test_normalization_with_delete(self):
        assert normalization.normalize(self.df5).equals(self.df2)